﻿using BGSMSEnums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;

namespace BGMS_ExampleCode {
    class Program {
        static void Main(string[] args) {
            string bgmsFile = convertToBGMS(args[0]);
            readBGMS(bgmsFile);
        }

        private static void readBGMS(string bgmsFile) {
            System.IO.FileInfo fInfo = new System.IO.FileInfo(bgmsFile);
            Console.WriteLine("Reading BGMS File: " + fInfo.Name);
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();
            //Opens a new BGMS file reader. 
            //The file reader is non-locking. This means that multiple readers can be open for the same file at the same time.
            BGSRawAPI.BGSRawFileReader reader = new BGSRawAPI.BGSRawFileReader(bgmsFile);

            MSMethodType methodType = reader.getMethodType();
            Dictionary<string, string> meta = reader.getMetaInfo();

            Console.WriteLine("Method: " + methodType.ToString());
            //Loop and print all attached meta information
            foreach (KeyValuePair<string, string> entry in meta) {
                Console.WriteLine(entry.Key + ": " + entry.Value);
            }

            //Traverses all scans in the BGMS file in the order they are stored.
            //The scans are numbered from first scan = 0 till last scan = getNrOfScans()-1
            double avgScanSizeInBytes = 0.0;
            int nrOfScans = reader.getNrOfScans();

            for (int scanIndex = 0; scanIndex < nrOfScans; ++scanIndex) {
                float[,] scan = reader.getScan(scanIndex);
                double rt = reader.getRT(scanIndex);
                bool isCentroid = reader.IsCentroidScan(scanIndex);

                avgScanSizeInBytes += scan.Length * sizeof(float);

                BGSRawAPI.PrecursorSelection[] quadSelection = reader.getPrecursorInfo(scanIndex);
            }

            reader.Close();
            stopwatch.Stop();

            avgScanSizeInBytes = avgScanSizeInBytes / (double)nrOfScans;

            Console.WriteLine("______________________________________________");
            Console.WriteLine("Total nr of scans: " +nrOfScans);
            Console.WriteLine("Avg scan size: " + Math.Round(avgScanSizeInBytes / 1024.0, 1) + " KB");
            Console.WriteLine("Time to read all scans: " + Math.Round((stopwatch.ElapsedMilliseconds / 1000.0), 2) + "s");
            Console.WriteLine("AVG time to read one scan: " + Math.Round((stopwatch.ElapsedMilliseconds / (double)nrOfScans), 3) + "ms");
        }

        private static string convertToBGMS(string rawFile) {
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();

            string outFile = rawFile.Replace(".raw", ".bgms");

            //our own wrapper class for the ThermoAPI
            ThermoAPIWrapper thermoReader = new ThermoAPIWrapper(rawFile);

            //Initialize a new BGMS file writer.
            //There are two different type of constructors for the BGSRawFileWriter.
            //Both work fine but the more elaborate one adds some fields that are important for SN QC perspective
            //Make sure the specify the MSMethod for optimal support in Spectronaut / SpectroMine / SpectroDive (HRM == DIA)
            BGSRawAPI.BGSRawFileWriter writer = new BGSRawAPI.BGSRawFileWriter("Thermo", thermoReader.getInstrumentName(), thermoReader.getInstrumentSerialNumber(), thermoReader.getRunDate(), MSMethodType.HRM, rawFile, outFile, false, true);

            writer.CustomOrder = null; //Optionally, one can apply a custom sort order for how the scans are ordered.
                                       //When using the MSMethodType HRM, this order will, by default, be set to the most ideal processing order for Spectronaut.
                                       //However, one can overwrite this to store the scans in RT or any other order
                                       //For the use in Spectronaut we only recommend using either the default or RT order.

            //Ideally one specifies the mass analyzer that where used to record this data.
            //Spectronaut, SpectronMine and SpectroDive will adapt some of the default parameter depending on the mass analyzer that was used.
            writer.MetaData[BGSRawAPI.BGSRawFileWriter.MS1_MASS_ANALYZER] = BGSMSEnums.MassAnalyzerType.Orbitrap.ToString();
            writer.MetaData[BGSRawAPI.BGSRawFileWriter.MS2_MASS_ANALYZER] = BGSMSEnums.MassAnalyzerType.Orbitrap.ToString();
            
            int nrOfScans = thermoReader.getTotalNumberOfScans();

            //just for indicating a progress in the console output
            int updateStep = (nrOfScans / 200);

            //loop over all scans (thermo scan indices start at 1)
            for (int scanIndex = 1; scanIndex <= nrOfScans; ++scanIndex) {

                //gets the MS-Level for a specific scan (MS1 or MS2)
                MSLevel msLevel = thermoReader.getMSLevelForScanNumber(scanIndex);

                //retrieves the retention time associated with this scan
                double rt = thermoReader.getRTFromScanNumber(scanIndex);

                //retrieves from the thermo API whether the scan in question was centroid or profile
                bool isCentroid = thermoReader.IsCentroidScan(scanIndex);


                //retrieves all quadrupole settings that where enabled to generate this scan.
                //If not using MSX, this always returns only 1 start and end m/z value
                List<BGSRawAPI.PrecursorSelection> precursorSelection = thermoReader.getQuatrupoleSelectionForScanNumber(scanIndex);


                //retrieves the actual scan data from the Thermo API
                double[,] scan = thermoReader.getScan(scanIndex);

                //Here one would process the scan if necessary. Cutting of baseline, centroiding etc.

                //Adds the scan to the BGMS file
                writer.AddScan(new BGSRawAPI.ScanEvent(scan, rt, msLevel, isCentroid, precursorSelection.ToArray()));

                //just some information on the console to indicate the progress
                if (scanIndex % updateStep == 0) {
                    Console.Write("\r                             \r" + scanIndex + " of " + nrOfScans);
                }
            }

            //closes the thermo API
            thermoReader.close();
            Console.WriteLine("");

            //closes and finalizes the BGMSWriter
            Console.WriteLine("Finalizing BGMS File...");
            writer.Finalize();

            stopwatch.Stop();
            Console.WriteLine("Time to convert raw file: " + Math.Round((stopwatch.ElapsedMilliseconds / 1000.0), 2) + "s"+Environment.NewLine);

            return outFile;
        }
    }
}
