﻿using BGSMSEnums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ThermoFisher.CommonCore.Data;
using ThermoFisher.CommonCore.Data.Business;
using ThermoFisher.CommonCore.Data.FilterEnums;
using ThermoFisher.CommonCore.Data.Interfaces;
using ThermoFisher.CommonCore.RawFileReader;

namespace BGMS_ExampleCode {
    public class ThermoAPIWrapper {
        private IRawDataPlus rawFileReader = null;

        public ThermoAPIWrapper(string file) {
            this.open(file);
        }

        private bool open(string filename) {
            if (this.rawFileReader == null) {
                IRawDataPlus rawFile = RawFileReaderAdapter.FileFactory(filename);
                rawFile.SelectInstrument(Device.MS, 1);
                this.rawFileReader = rawFile;
            }
            return this.rawFileReader != null && !this.rawFileReader.IsError;
        }

        public int getTotalNumberOfScans() {
            return this.rawFileReader.RunHeaderEx.SpectraCount;
        }

        public double getRTFromScanNumber(int scan) {
            return this.rawFileReader.RetentionTimeFromScanNumber(scan);
        }

        public bool IsCentroidScan(int scan) {
            return this.rawFileReader.IsCentroidScanFromScanNumber(scan);
        }


        public List<BGSRawAPI.PrecursorSelection> getQuatrupoleSelectionForScanNumber(int scan) {
            IScanEvent scanEvent = this.rawFileReader.GetScanEventForScanNumber(scan);
            List<BGSRawAPI.PrecursorSelection> massRanges = new List<BGSRawAPI.PrecursorSelection>();
            MSOrderType msOrder = scanEvent.MSOrder;

            if (msOrder == MSOrderType.Ms && scanEvent.MassCount == 0) {
                for (int i = 0; i < scanEvent.MassRangeCount; ++i) {
                    Range r = scanEvent.GetMassRange(i);
                    massRanges.Add(new BGSRawAPI.PrecursorSelection(r.Low, r.High));
                }
            } else {
                for (int i = 0; i < scanEvent.MassCount; ++i) {
                    try {
                        IReaction reaction = scanEvent.GetReaction(i);
                        double center = reaction.PrecursorMass;
                        double width = reaction.IsolationWidth;

                        double start = center - width / 2.0;
                        double end = start + width;

                        massRanges.Add(new BGSRawAPI.PrecursorSelection(start, end));
                    } catch { }
                }
            }

            return massRanges;
        }

        public double[,] getScan(int scanNr) {
            double[][] raw = null;
            ISimpleScanAccess scanObject = this.rawFileReader.GetSimplifiedScan(scanNr);

            raw = new double[2][];
            raw[0] = scanObject.Masses;
            raw[1] = scanObject.Intensities;

            double[,] scan = new double[2, raw[0].Length];

            System.Buffer.BlockCopy(raw[0], 0, scan, 0, sizeof(double) * raw[0].Length);
            System.Buffer.BlockCopy(raw[1], 0, scan, sizeof(double) * raw[0].Length, sizeof(double) * raw[0].Length);

            return scan;
        }

        public MSLevel getMSLevelForScanNumber(int scanNr) {
            IScanEvent scanEvent = this.rawFileReader.GetScanEventForScanNumber(scanNr);
            int msOrder = (int)scanEvent.MSOrder;
            return (MSLevel)(msOrder - 1);
        }

        public void close() {
            if (this.rawFileReader != null) {
                this.rawFileReader.Dispose();
                this.rawFileReader = null;
            }
        }

        public String getInstrumentName() {
            return this.rawFileReader.GetInstrumentData().Name;
        }

        public string getInstrumentModel() {
            return this.rawFileReader.GetInstrumentData().Model;
        }

        public string getInstrumentSerialNumber() {
            return this.rawFileReader.GetInstrumentData().SerialNumber;
        }

        public DateTime getRunDate() {
            return this.rawFileReader.CreationDate;
        }

        public BGSMSEnums.MassAnalyzerType getMassAnalyzerType(int scannr) {
            IScanEvent scan = rawFileReader.GetScanEventForScanNumber(scannr);
            return (BGSMSEnums.MassAnalyzerType)scan.MassAnalyzer;
        }
    }
}
